"use strict";
var ts = require("typescript");
var uml = require("./uml/index");
var Delinter = (function () {
    function Delinter() {
        this._umlProgram = new uml.Program();
    }
    Object.defineProperty(Delinter.prototype, "umlProgram", {
        get: function () {
            return this._umlProgram;
        },
        enumerable: true,
        configurable: true
    });
    Delinter.prototype.parse = function (file) {
        this._delintNode(file);
    };
    Delinter.prototype._delintNode = function (node) {
        var _this = this;
        switch (node.kind) {
            case ts.SyntaxKind.ClassDeclaration:
                this._delintClassDeclaration(node);
                break;
            case ts.SyntaxKind.InterfaceDeclaration:
                this._delintInterfaceDeclaration(node);
                break;
            default:
                break;
        }
        ts.forEachChild(node, function (n) { _this._delintNode(n); });
    };
    Delinter.prototype._delintClassDeclaration = function (node) {
        var _this = this;
        var umlClass = new uml.Class(node.name.getText());
        this._umlProgram.nodes.setValue(umlClass.name, umlClass);
        node.heritageClauses.forEach(function (h) {
            switch (h.token) {
                case ts.SyntaxKind.ImplementsKeyword:
                    h.types.forEach(function (t) {
                        var interfaceName = t.expression.getText();
                        if (!_this._umlProgram.nodes.containsKey(interfaceName)) {
                            var umlInterface = new uml.Interface(interfaceName);
                            _this._umlProgram.nodes.setValue(interfaceName, umlInterface);
                        }
                        var generalization = new uml.Generalization();
                        generalization.fromName = umlClass.name;
                        generalization.toName = interfaceName;
                        _this._umlProgram.generalizations.push(generalization);
                    });
                    break;
                case ts.SyntaxKind.ExtendsKeyword:
                    h.types.forEach(function (t) {
                        var parentClassName = t.expression.getText();
                        if (!_this._umlProgram.nodes.containsKey(parentClassName)) {
                            var umlParentClass = new uml.Class(parentClassName);
                            _this._umlProgram.nodes.setValue(parentClassName, umlParentClass);
                        }
                        var generalization = new uml.Generalization();
                        generalization.fromName = umlClass.name;
                        generalization.toName = parentClassName;
                        _this._umlProgram.generalizations.push(generalization);
                    });
                    break;
                default:
                    break;
            }
        });
    };
    Delinter.prototype._delintInterfaceDeclaration = function (node) {
        var umlInterface = new uml.Interface(node.name.getText());
        this._umlProgram.nodes.setValue(umlInterface.name, umlInterface);
    };
    return Delinter;
}());
exports.Delinter = Delinter;
